/********************************************
Copyright Pavel Potapov, 2013-2016
distributed under GNU General Public License
********************************************/

/******************************************************************
CALIBRATION FRAME
******************************************************************/	
//this claas is the calibration frame to make all image/beam shifts calibrated 
class ClickMoverCalibration112 : ChildUIFrame112
{
string Mode		//Mode = "Image" or "Beam"
string Axis		//Axis = "X" or "Y"

number ViewID	//ID of calibrated image
number ArrowID	//ID of arrow used for calibration

number IniImageShiftX,IniImageShiftY	//initial image shift kept as global values
number IniBeamShiftX,IniBeamShiftY		//initial beam shift kept as global values

number BeamImageCoupling	//how much image shift affect beam shift (depends on pivot accuracy
number BeamStepping		//steps of beam adjung

number ImageShiftApplied		//flag to know whether image shift apllied

number CurrentDeltaBeamShiftX,CurrentDeltaBeamShiftY	//"BeamChange" and "Next" methods both use these values

//draw arrow for calibration
void DrawArrow(object self,number alpha,number topD,number leftD)
	{
	image View := ViewID.getImageFromID()
 	number w,h
	getSize(View,w,h)

	number t,l,b,r	//relative coordinates
	//define top,left
	if((topD==0)&&(leftD==0)) 
		{
		//if this is first (X) axis
		t = 0.5 + 0.25*sin(alpha)
		l = 0.5 - 0.25*cos(alpha)
		}
	else 
		{
		//if this is 2nd axis (Y) - use information from positioned arrow in X-axis
		t=topD/h //default
		l=leftD/w //default
		}
	//define bottom,right
	b = t - 0.5*sin(alpha)
	r = l + 0.5*cos(alpha)
	
	//set arrow in absolute coordinates
	ArrowID = CreateArrowAnnotation(View,t*h,l*w,b*h,r*w) 
	}

//delete arrow for calibration	
void DeleteArrow(object self)
	{
	image View := ViewID.getImageFromID()
	try{DeleteAnnotation(View,ArrowID);} catch{break;}
	}

//Heading for calibration frame
string ConstructHead(object self)
	{
	string Head	= "CALIBRATE "+Mode+" SHIFT"
	Head += Axis
	
	return Head
	}

//DLG tags to make 4-arrow panel 	
TagGroup CreateQuatrePanel(object self,number size,number bcgr,number Rahm,TagGroup StepBeam)
	{	
	TagGroup Quatre = DLGCreateDialog("").DLGTableLayout(3,3,1)	

		number  w=1,h=1
		TagGroup DummyGraphic = DLGCreateGraphic(w,h)
	
	Quatre.DLGAddElement(DummyGraphic)
	Quatre.DLGAddElement(self.Button112(self.Iarrow(1,size,bcgr,Rahm),"BeamShift_Y_Plus"))		
	Quatre.DLGAddElement(DummyGraphic)	
	
	Quatre.DLGAddElement(self.Button112(self.Iarrow(2,size,bcgr,Rahm),"BeamShift_X_Menus").DLGAnchor("East"))
	Quatre.DLGAddElement(StepBeam)
	Quatre.DLGAddElement(self.Button112(self.Iarrow(0,size,bcgr,Rahm),"BeamShift_X_Plus").DLGAnchor("West"))
	
	Quatre.DLGAddElement(DummyGraphic)
	Quatre.DLGAddElement(self.Button112(self.Iarrow(3,size,bcgr,Rahm),"BeamShift_Y_Menus"))
	Quatre.DLGAddElement(DummyGraphic)
	
	return Quatre
	}
	
//DLG tags for calibration frame	
TagGroup CreateCalibration(object self,string Instruction,number Step)
	{	
	TagGroup CalibrationTags = DLGCreateDialog("")

	CalibrationTags.DLGAddElement(DLGCreateLabel(self.ConstructHead()+"X").DLGIdentifier("Head"))	
	
	CalibrationTags.DLGAddElement(DLGCreateLabel(Instruction))
	
		TagGroup ImageBox = DLGCreateBox("apply the specified image shift ")
		ImageBox.DLGTableLayout(3,1,0).DLGAnchor("West")						
		ImageBox.DLGAddElement(DLGCreateLabel("              "))	
		ImageBox.DLGAddElement(DLGCreateRealField(Step,8,2).DLGIdentifier("Step"))
		ImageBox.DLGAddElement(DLGCreatePushButton("   set   ", "SetImageShift").DLGIdentifier("SetImageShift"))
	CalibrationTags.DLGAddElement(ImageBox)	
	
	if(Mode=="Beam")
		{
		TagGroup BeamAlignBox = DLGCreateBox("correct caused beam shift")				
			number beamStep = round(Step/BeamStepping) 	//this is "BeamStepping" times less than Step
														//but may be refined manually
			TagGroup TBeamStep = DLGCreateIntegerField(beamStep,6).DLGIdentifier("BeamStep")			
		BeamAlignBox.DLGAddElement(self.CreateQuatrePanel(12,210,6,TBeamStep))

			TagGroup BeamShifts = DLGCreateDialog("").DLGTableLayout(2,2,1)
			number beamX,beamY
			ClickM112_GetBeamShift(beamX,beamY)						
			BeamShifts.DLGAddElement(DLGCreateLabel("beam shift X"))
			BeamShifts.DLGAddElement(DLGCreateIntegerField(beamX,6).DLGIdentifier("BeamX"))
			BeamShifts.DLGAddElement(DLGCreateLabel("beam shift Y"))
			BeamShifts.DLGAddElement(DLGCreateIntegerField(beamY,6).DLGIdentifier("BeamY"))		
		BeamAlignBox.DLGAddElement(BeamShifts)	
			
	CalibrationTags.DLGAddElement(BeamAlignBox)		
		}
		
	CalibrationTags.DLGAddElement(DLGCreatePushButton("NEXT...", "next"))
		
	return CalibrationTags
	}

//initiate X or Y axis to be calibrated	
void initAxis(object self,string AxisP)
	{
	axis = axisP
	self.LookUpElement("Head").DLGTitle(self.ConstructHead())
	}

//get scale of image in nm, correct if in um
number GetScale(object self,image im)
	{
	//get scale in nm
	number origin,scale
	string units
	ImageGetDimensionCalibration(im,0,origin,scale,units,1)
	if(!(units=="nm")) scale *= 1000	//image was probably calibrated in um (not nm)?
	
	return scale
	}

//calculate Step of imageShift such as it should be half-width of the live image in front	
number GetImageShiftStep(object self)
	{
	image View 
	try{View := getFrontImage();}
	catch {OKdialog("live View should be in front");break;}
	
	ViewID = View.getImageID()
	
	number scale = self.GetScale(View)
	number w,h
	getSize(View,w,h)

	number Step = round(w/2*scale)	//nm implied
//result("\nstep "+step+" "+w+" "+scale)	
	return Step
	}

//if image Shift already calibrated, the target value must be corrected	
number CorrectImageShiftStepForCalibration(object self,number Step,string Axis)
	{
	number cal = self.LoadNumber112(0,"Calibration "+"Image"+":calibration"+Axis)
	if(cal==0) throw("Image shift calibration not available !!")

	return cal*Step
	}
	
//Initialization	
void init(object self,number motherID,string ModeP,string Package)
	{
	Mode = ModeP
	self.SetPackageName(Package)
	
	BeamImageCoupling = self.LoadNumber112(0.2,"Beam-Image Coupling")
	BeamStepping = self.LoadNumber112(5,"Beam Stepping")
	self.StoreNumber112(BeamImageCoupling,"Beam-Image Coupling")
	self.StoreNumber112(BeamStepping,"Beam Stepping")
	
	//simple dialog with preliminary instruction
	string Instruction
	if(Mode=="Image")
		{
		Instruction = "Select the LIVE image.\n"
		Instruction += "Choose your typical magnification.\n" 
		Instruction += "find the recognisable feature" 
		}	
		
	if(Mode=="Beam")
		{
		//check if image shift calibration exist
		if(self.LoadNumber112(0,"Calibration "+"Image"+":axis standard")==0)
			{
			throw("Calibrate image shift first!!")
			}
		Instruction = "Look at Viewing screen.\n"
		Instruction += "Choose magnification 4-5 steps lower\n" 
		Instruction += "than your typical magnification."  
		}
	
	if(!OkCancelDialog(Instruction)) exit(0)
	
	//prepare image shift step 
	number Step  = self.GetImageShiftStep()
	
//prepare instruction to do calibration,draw arrow or correct step	
	if(Mode=="Image")
		{
		self.DrawArrow(0,0,0)
		
		Instruction = "Apply image shift and track the\nresulted jump by the arrow.\n\n" 
		Instruction += "You may shift image by microscope\n knobs for convenience.\n" 
		Instruction += "Microscope and DigitalMicrograph\n shifts are independent.\n" 
		Instruction += "\n" 
		Instruction += "Repeat few times\nto ensure accuracy\n"
		Instruction += "When ready press NEXT."	
		}
		
	if(Mode=="Beam")
		{
		Step = self.CorrectImageShiftStepForCalibration(Step,"X")
		Step /= BeamImageCoupling

		Instruction = "Center beam in viewing screen\nusing microscope tools.\n\n" 
		Instruction += "Apply image shift and recenter \nbeam by clicking arrow buttons.\n\n" 
		Instruction += "You might need to optimize steps\n of image and beam shifts\n\n" 
		Instruction += "When ready press NEXT."	
		}			
result("\nstep "+step)	
	//make calibration frame
	if(!self.initUIFChild112(motherID,self.CreateCalibration(Instruction,Step),Package,"Calibration")) throw("Cannot build calibration frame")
 
	self.initAxis("X") //start with X
	}

//Set/Reset Image shift is needed to calibrate the system response
void ResetImageShift(object self )
	{
	//SetImageShift must be fired prior at least once to set IniImageShiftX,IniImageShiftY values
result("\niniImShiftX "+IniImageShiftX+" iniImShiftY "+IniImageShiftY	)	

	ClickM112_SetImageShift(IniImageShiftX,IniImageShiftY)	
	if(Mode=="Beam") 
		{
		ClickM112_SetBeamShift(IniBeamShiftX,IniBeamShiftY)	
result("\niniBeamShiftX "+IniBeamShiftX+" iniBeamShiftY "+IniBeamShiftY	)
		}
		
	ImageShiftApplied = 0
	
	self.LookUpElement("SetImageShift").DLGTitle("  set  ")
	}

//Set/Reset Image shift is needed to calibrate the system response	
void SetImageShift(object self )
	{	
	if(ImageShiftApplied ==0)
		{
		ClickM112_GetImageShift(IniImageShiftX,IniImageShiftY)	//get initial ABSOLUTE value before adding image shift
		if(Mode=="Beam") ClickM112_GetBeamShift(IniBeamShiftX,IniBeamShiftY) //set initial ABSOLUTE value before adding beam shift	

		number Step = self.LookUpElement("Step").DLGGetValue()
		number shiftX=0,shiftY=0
		if(axis=="X") shiftX = Step
		if(axis=="Y") shiftY = Step
		
		ClickM112_ChangeImageShift(ShiftX,ShiftY)	
result("\nImageShiftX "+ShiftX+" ImageShiftY "+ShiftY	)		
		self.LookUpElement("SetImageShift").DLGTitle("reset")
		
		ImageShiftApplied = 1
		}
	else
		{
		self.ResetImageShift()	
		}	
	}	


/***************These methods group makes beam shift*****************/

void BeamChange(object self,string Ax,number Direction)
	{
	if(	ImageShiftApplied==0) throw("Apply Image Shift first !!")
	
	number X=0,Y=0
	if(Direction!=1) if(Direction!=(-1)) throw("Direction should be 1 or -1 !!")
	if(Ax=="X") X=Direction
	if(Ax=="Y") Y=Direction	
	
	number Step = self.LookUpElement("BeamStep").DLGGetValue()
	ClickM112_ChangeBeamShift(X*Step,Y*Step)
		
	CurrentDeltaBeamShiftX += (X*Step)
	CurrentDeltaBeamShiftY += (Y*Step)
	
	self.RefreshDLGValue112(self.LookUpElement("BeamX"),X*Step)
	self.RefreshDLGValue112(self.LookUpElement("BeamY"),Y*Step)
	}
		
void BeamShift_X_Plus(object self) self.BeamChange("X",1)
void BeamShift_X_Menus(object self) self.BeamChange("X",-1)
void BeamShift_Y_Plus(object self) self.BeamChange("Y",1)
void BeamShift_Y_Menus(object self) self.BeamChange("Y",-1)


/********************calculate and set calibration*****************/

/* these functions analyses  shift AppliedOn = "onX" or "onY" and compares it with measured response: onX, onY 
XonX - how much it is shifted on X when shiftX0 applied
YonX - how much it is shifted on Y when shiftX0 applied
YonY - how much it is shifted on Y when shiftY0 applied
XonY - how much it is shifted on X when shiftY0 applied
*/
	
number calibrateImageShift(object self,number input,number respX,number respY,number scale)
	{
	if(!((respX==0)&&(respY==0)||(input==0)))
		{
		//this calculates the MODULUS of applied vslue
		//divided by MODULUS of response 

		return modulus(input) / (sqrt(respX**2+respY**2)*scale ) 																	
		}
	else return 1
	}

void CalibrateBeamShiftValues(object self,number &calRespX,number &calRespY,number stepX,number stepY,number XonX,number XonY,number YonX,number YonY)
	{
	//this calculates stretching in responsed X and Y provided that inputX and Y is applied
	//solving system of equations
	//	inputX**2 = stretchX**2*XonX + stretchY**2*XonY
	//	inputY**2 = stretchX**2*YonX + stretchY**2*YonY
	
	image A:= realimage("A",4,2,2)
	image B:= realimage("B",4,1,2)
	
	A.setPixel(0,0,XonX**2)
	A.setPixel(1,0,XonY**2)
	A.setPixel(0,1,YonX**2)
	A.setPixel(1,1,YonY**2)
	
	B.setPixel(0,0,stepX**2)
	B.setPixel(0,1,stepY**2)
//A.showimage()	
//B.showimage()	

	image X = MatrixMultiply(MatrixInverse(A),B)
//X.showimage()		
	calRespX = X.getPixel(0,0)
	calRespY = X.getPixel(0,1)
	if(calRespX>=0) calRespX = sqrt(calRespX)
	else calRespX = 0
	if(calRespY>=0) calRespY = sqrt(calRespY)
	else calRespY = 0
	}
	
//collect all information about reponse, converts it in calibration values and store
void CalculateAndSetCalibration(object self,string Mode)
	{
	//This calculate parameters to go from initial to final coord systems
	//"Image" mode : from 'image' coordinates to 'microscope image shift' coordinates
	//"Beam" mode : from 'microscope image shift' coordinates to 'microscope beam shift' coordinates
		
result("\ncalibration" +Mode+ " shift")
	
	//read from temporary response numbers
	number XonX = self.LoadNumber112(1,"response:"+Mode+":X:onX")
	number XonY = self.LoadNumber112(1,"response:"+Mode+":X:onY")
	number stepX = self.LoadNumber112(1,"response:"+Mode+":X:step")
	number YonX = self.LoadNumber112(1,"response:"+Mode+":Y:onX")
	number YonY = self.LoadNumber112(1,"response:"+Mode+":Y:onY")
	number stepY = self.LoadNumber112(1,"response:"+Mode+":Y:step")
	
	//get scale
	number scale 
	image View := ViewID.getImageFromID()
	if(Mode=="Image") scale = self.GetScale(View)
	if(Mode=="Beam") scale = 1

	//calculate calibration values
	number calInputX,calInputY
	number calRespX,calRespY
	if(Mode=="Image")
		{
		//input is applied in final coord system
		calInputX = self.calibrateImageShift(StepX,XonX,XonY,scale)
		calInputY = self.calibrateImageShift(StepY,YonX,YonY,scale)
		
		//response is in initial system
		//image is assumed already well calibrated
		calRespX = 1
		calRespY = 1
		}
	if(Mode=="Beam")
		{
		//input is applied in initial system
		//image shift is assumed already well calibrated
		calInputX = self.LoadNumber112(1,"Calibration "+"Image"+":calibrationX")
		calInputY = self.LoadNumber112(1,"Calibration "+"Image"+":calibrationY")
		stepX *= calInputX
		stepY *= calInputY
		
		//response is in final system, stretching factors are mixed in X,Y-responses
		self.CalibrateBeamShiftValues(calRespX,calRespY,stepX,stepY,XonX,XonY,YonX,YonY)
		
		//correct beamShift response
		XonX /= calRespX
		XonY /= calRespY
		YonX /= calRespX
		YonY /= calRespY
		}	
	
	//Note that imageShift and BeamShift can be defined 
	//in various coord system in the specific microscope
	//stnd = +1 : x-right,y-up
	//stnd = -1 : x-right,y-down
	//first of all one need to gues if initial and final systems are in the same standard
	number same = CheckCoordinatesStandard112(XonX,XonY,YonX,YonY)
	if(same==0) throw("At least one coordinate system seems to be non-orthogonal")
result("\nsame ("+same+")")	

	//knowing the standard of one system, deduce it for another one
	number stnd0,stndF
	if(Mode=="Image") 
		{
		//imageShift system -> image system
		stndF = -1	//Gatan image coordinates is y-down
		if(same==1) stnd0 = stndF
		if(same==(-1))	stnd0 = -stndF
		}
	if(Mode=="Beam") 
		{
		//imageShift system -> BeamShift system
		stnd0 = self.LoadNumber112(1,"Calibration "+"Image"+":axis standard")
		if(same) stndF = stnd0
		if(same==(-1))	stndF = -stnd0
		}
result("\nstnd0 "+stnd0+" stndF "+stndF)		

	//calculate rotation angle of final  system relative the initial one
	//angle is defined always between X-axis, no matter what is the system standard 
	//if standards are determined correctly, angles determined from X and Y axis must be ~same
	number alphaX = GetRotationAngle112("onX",XonX,stndF*XonY)	
	number alphaY = GetRotationAngle112("onY",stnd0*YonX,stnd0*stndF*YonY)
result("\nX deduced alpha "+alphaX)
result("\nY deduced alpha "+alphaY)	
	//average for better accuracy
	number alphaAv = (alphaX+alphaY)/2
result("\naveraged alpha "+alphaAv)

	//prepare parameters for the final coordinate system (initial is not needed)
	number alpha,calX,calY,stnd
		if(Mode=="Image") 
			{
			//because here input and response are reverted
			alpha = alphaAv	
			calX = calInputX
			calY = calInputY
			stnd = stnd0
			}
			
		if(Mode=="Beam") 
			{
			//because here input and response are reverted
			alpha = -alphaAv	
			calX = 1/calRespX
			calY = 1/calRespY
			stnd = stndF
			}
result("\nalpha "+alpha+" calX "+calX+" calY "+calY+" stnd "+stnd)		

	//store final calibration numbers
	self.StoreNumber112(alpha,"Calibration "+Mode+":alpha")
	self.StoreNumber112(stnd,"Calibration "+Mode+":axis standard")
	self.StoreNumber112(calX,"Calibration "+Mode+":calibrationX")
	self.StoreNumber112(calY,"Calibration "+Mode+":calibrationY")
	}

//measures the response to applied shifts, stores it and switch to another axis	
void next(object self )
	{	
	self.resetImageShift()

	number dX,dY

	image View := ViewID.getIMageFromID()
	
	//calculate temporary response numbers for two modes
	if(Mode=="Image")
		{
		number top,left,bottom,right
		getannotationrect(View,ArrowID,top,left,bottom,right)
		DeleteAnnotation(View,ArrowID) 
		dX = right-left
		dY = bottom-top 
		
		//estimate alpha for the Y arrow 
		if((Mode=="Image")&&(axis=="X"))
			{
			number alphaX = atan2(top-bottom,right-left)
			number alphaY = alphaX+1.57	//true only when rotated system is standard, if not - opposite
			self.DrawArrow(alphaY,top,left)	//suggest the estimated alpha value
			}
		}	
	if(Mode=="Beam") 
		{
		dX =  -CurrentDeltaBeamShiftX	//because it is measured by compensation
		dY =  -CurrentDeltaBeamShiftY
		
		CurrentDeltaBeamShiftX = 0		
		CurrentDeltaBeamShiftY = 0	
		}
	
	//These temporary response number should be stored; CalculateAndSetCalibration then read it 
	self.StoreNumber112(dX,"response:"+Mode+":"+Axis+":onX")
	self.StoreNumber112(dY,"response:"+Mode+":"+Axis+":onY")
			number Step = self.LookUpElement("Step").DLGGetValue()
	self.StoreNumber112(step,"response:"+Mode+":"+Axis+":step")
	
	//decide whether continue with Y-axis or finish
	if(axis=="X") //continue with Y
		{			
		self.initAxis("Y");	
		if(Mode=="Beam") 
			{
			//refresh ImageShift step value
			number Step  = self.GetImageShiftStep()
			Step = self.CorrectImageShiftStepForCalibration(Step,"Y")
			Step /= BeamImageCoupling
			
			//refresh BeamShift step value
			self.lookUpElement("Step").DLGValue(Step)
			self.LookUpElement("BeamStep").DLGValue(round(Step/BeamStepping))
			}
		}
	else 	//calculate and store caclibration
		{
		self.CalculateAndSetCalibration(Mode)

		number XonX = self.LoadNumber112(1,"response:"+Mode+":X:onX")
		number XonY = self.LoadNumber112(1,"response:"+Mode+":X:onY")
		number YonX = self.LoadNumber112(1,"response:"+Mode+":Y:onX")
		number YonY = self.LoadNumber112(1,"response:"+Mode+":Y:onY")
		OKdialog(Mode+ "response\nXonX:"+XonX+"\nXonY:"+XonY+"\nYonX:"+YonX+"\nYonY:"+YonY+"\ncalibration is stored")
		
		self.SafeCloseUIFrame112()
		}
	}
	
number AboutToCloseUIFrame112(object self, number verify)
	{
	self.DeleteArrow()
	
	return 1
	}
}

/******************************************************************
CLICKMOVER SETTINGS
******************************************************************/	
//this class is settings frame that allows to lanch image and beam calibrations
class ClickMoverSettings112 : ChildUIFrame112
{

//DLG tags	
TagGroup SettingsParam(object self)
	{
	TagGroup Param, ParamItems
	Param = DLGCreateDialog("Settings",ParamItems)
	Param.DLGTableLayout(2,3,0).DLGAnchor("West")
	

	ParamItems.DLGAddElement(DLGCreateLabel("     image shift").DLGAnchor("East"))
	ParamItems.DLGAddElement(DLGCreatePushButton("calibrate","CalibrateImageShift"))

	ParamItems.DLGAddElement(DLGCreateLabel("    beam shift").DLGAnchor("East"))
	ParamItems.DLGAddElement(DLGCreatePushButton("calibrate","CalibrateBeamShift"))
	
	ParamItems.DLGAddElement(DLGCreatePushButton("Close","CloseSettings").DLGAnchor("East"))
	
	return Param
	}

//Initialization	
void init(object self,number motherID,string Package)
	{				
	self.initUIFChild112(motherID,self.SettingsParam(),Package,"Settings")
	}

//start image shift calibration	
void CalibrateImageShift(object self)
	{
	self.AllocFromDummyObject("CalibrationFrame").init(self.ScriptObjectGetID(),"Image",self.GetPackageName())
	}

//start beam shift calibration	
void CalibrateBeamShift(object self)
	{
	self.AllocFromDummyObject("CalibrationFrame").init(self.ScriptObjectGetID(),"Beam",self.GetPackageName())
	}

//This is needed just to close UIFrame	
void CloseSettings(object self)
	{						
	self.SafeCloseUIFrame112()
	}
	
}

/******************************************************************
CLICK MOVER attached to image
******************************************************************/	
//this class define the object attached to a given image and reacted on Mouse double-click
Class ClickMoverAttached112
{
number t0		//time mark to track tiem intervals
number HRtick	//number of HighResolution ticks for 1 sec in the system

number width,height	//needed as global to find the image center

number ApplyBeam	//apply or not beam correction

number ImageAlpha,ImageStnd,ImageCalibrX,ImageCalibrY	//all calibration values as global
number BeamAlpha,BeamStnd,BeamCalibrX,BeamCalibrY		//loaded in object once initiated

image View		//assigned image
string package	//needed for loading parameters

//This is analogeous to UIFrame112, Frame112 objects
string GetPackageName(object self) return Package

//Initialization	
void init(object self,image viewP,number ApplyBeamP,number HRtickP,string packageP)
	{
	package = packageP
	ApplyBeam = ApplyBeamP
	HRtick = HRtickP
	View:=ViewP
	
	//loading calibrations
	ImageAlpha = self.LoadNumber112(0,"Calibration "+"Image"+":alpha")
	ImageStnd = self.LoadNumber112(1,"Calibration "+"Image"+":axis standard")
	ImageCalibrX = self.LoadNumber112(0,"Calibration "+"Image"+":calibrationX")
	ImageCalibrY = self.LoadNumber112(0,"Calibration "+"Image"+":calibrationY")

	BeamAlpha = self.LoadNumber112(0,"Calibration "+"Beam"+":alpha")
	BeamStnd = self.LoadNumber112(1,"Calibration "+"Beam"+":axis standard")
	BeamCalibrX = self.LoadNumber112(0,"Calibration "+"Beam"+":calibrationX")
	BeamCalibrY = self.LoadNumber112(0,"Calibration "+"Beam"+":calibrationY")

//result("\nImage alpha "+ImageAlpha+" stnd "+ImageStnd+" calibrX "+ImagecalibrX+" calibrY "+ImagecalibrY)
//result("\nBeam alpha "+BeamAlpha+" stnd "+BeamStnd+" calibrX "+BeamcalibrX+" calibrY "+BeamcalibrY)
	
	getsize(View,width,height)
result("\nClickMoverAttached "+self.ScriptObjectgetID()+" initialised")
	}

//Destructor	
~ClickMoverAttached112(object self)
	{
result("\nClickMoverAttached "+self.ScriptObjectgetID()+" destroyed")		
	}

//correct beamShift in response to imageShift 
//(it can be fired also from the Starter)
void CorrectBeamShift(object self,number ImShDeltaX,number ImShDeltaY)
	{
	number BmShDeltaX,BmShDeltaY
	RotateCoordinates112(BmShDeltaX,BmShDeltaY,BeamAlpha,Imagestnd,Beamstnd,ImShDeltaX,ImShDeltaY)
//result("\n"+BmShDeltaX+" "+BmShDeltaX+" "+BmShDeltaY+" "+BmShDeltaY)
	
	BmShDeltaX *= BeamCalibrX
	BmShDeltaY *= BeamCalibrY
	
	ClickM112_ChangeBeamShift(-BmShDeltaX,-BmShDeltaY)
	}

//compenstate shift in image by imageshift in microscope
void CompensateImageShift(object self,number DeltaX,number DeltaY)
	{
	number incrShX,incrShY
	RotateCoordinates112(incrShX,incrShY,ImageAlpha,-1,Imagestnd,DeltaX,DeltaY)
//result("\njump x" +jumpX+" y "+jumpY+" incX "+incrShX+" incY "+incrShY)
	
	incrShX *= ImageCalibrX
	incrShY *= ImageCalibrY
	
	//compensate
	ClickM112_ChangeImageShift(-incrShX,-incrShY) 	
	
	//correct beam shift
	if(ApplyBeam==1) 
		{
		self.CorrectBeamShift(-incrShX,-incrShY)
		}
	}
	
//THis method apply the image shift such as the mouse-clicked point moves to the image center	
void MoveToCenter(object self,number x, number y,number scale)
	{
	//deviations from image center (all in standard coordinate system)
	number jumpX = (x  - width/2)*scale		//in nm
	number jumpY = (y - height/2)*scale		//in nm

	//move to center
	self.CompensateImageShift(jumpX,jumpY)
	}

//This tracks the mouse double-click event	
void MouseDoubleClick(object self, Number e_fl, ImageDisplay idisp, Number r_fl, Number r_fl2, ROI r )
	{
	if(r.ROIGetname() != "ClickROI") return // Verify it is the proper ROI

	//Check double click
	number maxdt = 0.3	//double click must occur within maxdt (in sec)
	number t = GetHighResTickCount()	
	if((t-t0)<maxdt*HRtick) 
		{
		// Get the mouse position
		Number mx, my 
		DocumentWindow win = idisp.ImageDisplayGetImage().ImageGetOrCreateImageDocument().ImageDocumentGetWindow()
		win.WindowGetMousePosition(mx,my)
//Result("\nMouse click at ("+mx+","+my+") t "+t)		
		// Compute the mouse-coordinates as image coordinates
		Number C2W_ox, C2W_oy, C2W_sx, C2W_sy,X,Y
		idisp.ComponentGetChildToWindowTransform(C2W_ox, C2W_oy, C2W_sx, C2W_sy)
		X = trunc((mx - C2W_ox) / C2W_sx)
		Y = trunc((my - C2W_oy) / C2W_sy)
		
//Result("\nMouse click at ("+X+","+Y+") t "+t)

		//get scale in nm
		number origin,scale
		string units
		ImageGetDimensionCalibration(view,0,origin,scale,units,1)
		if(!(units=="nm")) scale *= 1000	//image was probably calibrated in um (not nm)?
		
		//fire moving
		self.MoveToCenter(x,y,scale)
		}		
	t0=t	//reset time mark
	}
}


/******************************************************************
ClickMover STARTER
******************************************************************/	
//this class define the starter frame for ClickMover allowing to assign/deassignimage, reset image shift et cet.
class ClickMover112 :UIFrame112
{
number ViewID		//ID of assigned image
number roiID		//ID of all-field-of-view ROI in it

number ViewListenerID	//Listeners ID
number ViewWindowID

number HRtick	//number of HighResolution ticks for 1 sec in the system

number IniImageShiftX,IniImageShiftY	//initial(before ClickMover manipulations) values of image shift
number IniBeamShiftX,IniBeamShiftY	//initial(before ClickMover manipulations) values of image shift
	
object ClickMover 	//ClickMoverAttached object to be attached to image	
object DummySettings	//dummy objects to keep clasess
object DummyCalibration	

//DLG tags	
TagGroup CreateStarter(object self)
	{		
	number Bcgrd  =210
	number size =20

	TagGroup Starter = DLGCreateBox("")
	
	Starter.DLGTableLayout(3,2,0).DLGAnchor("West")

	Starter.DLGAddElement(DLGCreatePushButton("assign View","Assign"))
	Starter.DLGAddElement(DLGCreateStringField("",10).DLGIdentifier("ImageName"))
	Starter.DLGAddElement(DLGCreatePushButton("de-assign","deAssign"))

	Starter.DLGAddElement(DLGCreateCheckbox("shift beam",0).DLGIdentifier("ApplyBeam")).DLGAnchor("East")	
	Starter.DLGAddElement(DLGCreatePushButton("  reset    ","ResetShift")).DLGAnchor("West")
	Starter.DLGAddElement(self.SettingsButton(size,Bcgrd,"Parameters")).DLGAnchor("East") //SettingsButton extFrame function
	
	return Starter
	}


//Initialization	
void init(object self,string Package)
	{ 			
	self.initUIFrame112(self.CreateStarter(),Package,"Starter")

	//check the number of HighResolution ticks for 1 sec in the system
	HRtick = self.LoadNumber112(0,"HRtick 1sec")
	if(HRtick==0)
		{
		//measure HighResolution ticks
result("\nmeasuring HRtick of the system...")
		number t0 = GetHighResTickCount()
		sleep(1)
		HRtick = GetHighResTickCount()-t0
result(" "+HRtick+" per sec")	
		self.StoreNumber112(HRtick,"HRtick 1sec")
		}
	
	//set applyBeam value
	self.LookUpElement("ApplyBeam").DLGvalue(self.LoadNumber112(0,"ApplyBeamShift"))
	
	//allocate ClickMoverattached (May be attached to only one Imaage at once)
	ClickMover = alloc(ClickMoverAttached112)

	//store UIFrame classes	
	dummySettings = self.KeepClassInDummyObject(alloc(ClickMoverSettings112),"ClickMoverSettings")
result("\ndummy ClickMoverSettings "+ dummySettings .ScriptObjectgetID() +" stored")
	dummyCalibration = self.KeepClassInDummyObject(alloc(ClickMoverCalibration112),"CalibrationFrame")
result("\ndummy CalibrationFrame "+ dummyCalibration .ScriptObjectgetID() +" stored")

	//remember initial image shifts
	ClickM112_GetImageShift(IniImageShiftX,IniImageShiftY)	
	ClickM112_GetBeamShift(IniBeamShiftX,IniBeamShiftY)		
	}
	
void closeUIFrame112(object self)
	{
	result ("\nhere")
	}	 
	
//assign ClickMoverAttached to image
void Assign(object self)
	{
	number ApplyBeam = self.LookUpElement("ApplyBeam").DLGGetValue()
		
	//check if calibration available
	number ImageCalibrX = self.LoadNumber112(0,"Calibration "+"Image"+":calibrationX")
	if(ImageCalibrX==0) throw("Image shift calibration is ZERO\nrun image shift calibration first")
	number BeamCalibrX = self.LoadNumber112(0,"Calibration "+"Beam"+":calibrationX")
	if(ApplyBeam==1)
		{
		if(BeamCalibrX==0) 
			{
			if(!OKcancelDialog("Beam calibration is ZERO !!\nCalibrate beam shift first")) exit(0)
			else (OKdialog("Shift beam not applied"))
			self.LookUpElement("ApplyBeam").DLGValue(0)
			}
		}
	
	//get front image as target
	image View:=getFrontImage()
	ViewID = View.ImageGetID()
	self.LookUpElement("ImageName").DLGValue(View.ImageGetName())

	//prepare all-field-of-view ROI in image (needed to track mouse)
	ImageDisplay idisp = View.ImageGetImageDisplay(0)	
	roi FOVroi = NewROI()
	FOVroi.ROISetVolatile(0)
	FOVroi.ROISetResizable(0)
	FOVroi.ROISetDeletable(0)
	FOVroi.ROISetName("ClickROI")
	FOVroi.ROISetDeletable(0)
	FOVroi.ROISetcolor(1,1,1)
	roiID = FOVroi.RoiGetID()	
	number width,height
	view.GetSize(width,height)
	FOVroi.ROISetRectangle(0,0,width,height)	
	idisp.ImageDisplayAddRoi(FOVroi)
	
	//attach ClickMoverAttached to View
	ClickMover.init(View,ApplyBeam,HRtick,self.GetPackageName())

	//make listeners to mouse-click and closed window
	ViewListenerID = idisp.ImageDisplayAddEventListener(ClickMover, "roi_end_track:MouseDoubleClick")
	imagedocument Viewdoc = View.ImageGetorCreateImageDocument()
	ViewWindowID = Viewdoc.ImageDocumentGetWindow().WindowAddWindowListener(self,"window_closed:ClosedView")
	}

void ResetShift(object self)
	{	
	ClickM112_SetImageShift(IniImageShiftX,IniImageShiftY)
	ClickM112_SetBeamShift(IniBeamShiftX,IniBeamShiftY)
	}
	
//reset all note and IDs related with image
void ResetView(object self)
	{
	ViewID = 0
	ViewListenerID = 0
	ViewWindowID = 0
	
	self.LookUpElement("ImageName").DLGValue("")
	}

//de-assign ClickMoverAttached from image	
void deAssign(object self)
	{
	image View := ViewID.FindImageByID()
	imagedocument Viewdoc = View.ImageGetorCreateImageDocument()
	if(Viewdoc.CheckImageWindow112())
		{
		imageDisplay imdisp = View.ImageGetImageDisplay(0)
		imdisp.ImageDisplayDeleteROI(roiID.GetRoiFromID())	
		imdisp.ImageDisplayRemoveEventListener(ViewListenerID)	
	
		Viewdoc.ImageDocumentGetWindow().WindowRemoveWindowListener(ViewWindowID)	
		}
	
	self.ResetView()
	self.ResetShift()
	}

void ClosedView(object self, number e_fl, DocumentWindow Win)
	{
	self.ResetView()
	}
		
number AboutToCloseUIFrame112(object self,number verify)
	{
	number ApplyBeam = self.LookUpElement("ApplyBeam").DLGGetValue()
	self.StoreNumber112(ApplyBeam,"ApplyBeamShift")
	
	self.deAssign()	
	
	return 1
	}
				
void Parameters(object self )
	{
	self.AllocFromDummyObject("ClickMoverSettings").init(self.ScriptObjectGetID(),self.GetPackageName())
	}	

//This reset all USER shifts applied in course of functioning this package	

}
 

void CreateAll()  
{
//check in-Line
string checkInLine1= "emimageshift"
string checkInLine2 = "emgetimageshift"
	
if(!doesfunctionexist(checkInLine1)&&!doesclassexist(checkInLine2))
	{
	throw("\ntemDM temDM ClickMover cannot detect microscope control\nif this is the off-line GMS, install 'temDM virtualTEM'")
	}
	
string Package = "_temDM-Scripts:ClickMover"

alloc(ClickMover112).init(Package);		
}

CreateAll()


