/********************************************
Copyright Pavel Potapov, 2013-2016
distributed under GNU General Public License
********************************************/

/***********************CHECK********************************/

if(doesfunctionexist("rotatecoordinates112"))
	{
	result("\nfunctions "+"Rotation112"+" already installed\n");
	throw("")
	}	
result("\ninstall functions "+"Rotation112")


/******************************************************************
function : Rotate coordinates
******************************************************************/
void RotateCoordinates112(number &x,number &y,number alpha,number stnd0,number stnd,number x0,number y0)
//rotated coordinates (x,y) from initial coordinates(x0,y0) 
//alpha(rad) is counted COUNTCLOCKWISE, stnd=1 for x-right,y-up ; stnd=(-1) for x-right,y-down
{
number XX,XY,YX,YY
XX = 1
XY = stnd0
YX = -stnd
YY = -stnd0*YX

x = XX*cos(alpha)*x0+XY*sin(alpha)*y0
y = YX*sin(alpha)*x0+YY*cos(alpha)*y0
}

/******************************************************************
function : Get rotation angle
******************************************************************/
number GetRotationAngle112(string AppliedOn,number delX,number delY)
{ 
//rotstion angle deduced from input along X, or Y in initial coord system 
//causing response in rotated (x,y) coord system
//standart axes counterclock: 0...pi; clockwise: 0...-pi

number horiz,vert
if(AppliedOn=="onX") {vert = delY;horiz = delX;}	
if(AppliedOn=="onY") {vert = -delX;horiz = delY;}
//both ini and rotated coord system assumed to be standard (x-right,y-up)

return atan2(vert,horiz)	//-pi...+pi
}

/******************************************************************
function : check if ini and rotated coord systems are in the same standard
******************************************************************/
number CheckCoordinatesStandard112(number XonX,number XonY,number YonX,number YonY)
{ 
//positive input onX in ini system -> response XonX, XonY in rotated system
//positive input onY in ini system -> response YonX, YonY in rotated system

number alphaX = GetRotationAngle112("onX",XonX,XonY)
number alphaY = GetRotationAngle112("onY",YonX,YonY)
result("\ntest X alpha "+alphaX)
result("\ntest Y alpha "+alphaY)

number same
number maxNonOrthogonality = 10*2*pi()/180 //10grad imperfection for each axis allowed
if(modulus(alphaY-alphaX)<maxNonOrthogonality) same = 1	
if(modulus( modulus(alphaY-alphaX)-pi() )<maxNonOrthogonality ) same = (-1)	
					//if alphaX-alphaY = pi then system are defined in different standards

return same
}

/******************************************************************
function : Rotate image with keeping original size
******************************************************************/
image Rotate112(image im, number angle)
	{
	//rotation angle in grad
	//0...+90 countclockwise
	//0...-90 clockwise
	
	//get contrast limits
	number low,high
	getlimits(im,low,high)
	
	//rotation numbers
	number RadAngle = angle/(180/pi())
	number CT = cos(RadAngle)
	number ST = sin(RadAngle)	

	//rotate	
	image rotim:=rotate(im,RadAngle)

	//get dimensions
	number W,H,rW,rH
	getSize(im,W,H)
	getSize(rotim,rW,rH)
//result("\n"+" W "+w+"x"+h+" "+rW+"x"+rH)

	//define rectangular
	number top,left,bottom,right
	top = rH/2 - H/2
	bottom = rH/2 + H/2
	left = rW/2 - W/2
	right = rW/2 + W/2

	//clip
	image out := im.imageclone()
	out = 0
	out = rotim[top,left,bottom,right]	
	
	//set background
	out = tert((modulus((icol-w/2)*CT-(irow-h/2)*ST))< w/2,out,high)	
	out = tert((modulus((icol-w/2)*ST+(irow-h/2)*CT))< h/2,out,high)
			
	return out
	}

